"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useFontSize = exports.useLayout = exports.useMediaQuery = exports.useFocus = exports.useActive = exports.useHover = exports.useScreenSize = void 0;
/* eslint-disable react/display-name */
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const convertUnits_1 = require("./convertUnits");
const mediaQueries_1 = require("./cssToRN/mediaQueries");
/** Hook that will apply the screen size to the styles defined with vmin, vmax, vw, vh units, and handle media queries constraints */
const useScreenSize = () => {
    const { width, height } = (0, react_native_1.useWindowDimensions)();
    return react_1.default.useMemo(() => ({
        vw: width / 100,
        vh: height / 100,
        vmin: Math.min(width, height) / 100,
        vmax: Math.max(width, height) / 100,
        dvw: width / 100,
        dvh: height / 100,
        lvw: width / 100,
        lvh: height / 100,
        svw: width / 100,
        svh: height / 100
    }), [height, width]);
};
exports.useScreenSize = useScreenSize;
/** Hook that will apply the style reserved for hover state if needed */
const useHover = (onMouseEnter, onMouseLeave, needsHover) => {
    const [hover, setHover] = react_1.default.useState(false);
    const hoverStart = react_1.default.useMemo(() => needsHover ? (event) => {
        if (onMouseEnter)
            onMouseEnter(event);
        setHover(true);
    } : undefined, [needsHover, onMouseEnter]);
    const hoverStop = react_1.default.useMemo(() => needsHover ? (event) => {
        if (onMouseLeave)
            onMouseLeave(event);
        setHover(false);
    } : undefined, [needsHover, onMouseLeave]);
    return { hover, onMouseEnter: hoverStart || onMouseEnter, onMouseLeave: hoverStop || onMouseLeave };
};
exports.useHover = useHover;
/** Hook that will apply the style reserved for active state if needed */
const useActive = (onPressIn, onPressOut, onResponderStart, onResponderRelease, onStartShouldSetResponder, needsTouch) => {
    const [active, setActive] = react_1.default.useState(false);
    const touchStart = react_1.default.useMemo(() => needsTouch ? (event) => {
        if (onPressIn)
            onPressIn(event);
        else if (onResponderStart)
            onResponderStart(event);
        setActive(true);
    } : undefined, [needsTouch, onResponderStart, onPressIn]);
    const touchEnd = react_1.default.useMemo(() => needsTouch ? (event) => {
        if (onPressOut)
            onPressOut(event);
        else if (onResponderRelease)
            onResponderRelease(event);
        setActive(false);
    } : undefined, [needsTouch, onResponderRelease, onPressOut]);
    const grantTouch = react_1.default.useMemo(() => needsTouch ? onStartShouldSetResponder || (() => true) : undefined, [needsTouch, onStartShouldSetResponder]);
    return {
        active,
        onPressIn: touchStart || onPressIn,
        onPressOut: touchEnd || onPressOut,
        onResponderStart: touchStart || onResponderStart,
        onResponderRelease: touchEnd || onResponderRelease,
        onStartShouldSetResponder: grantTouch || onStartShouldSetResponder
    };
};
exports.useActive = useActive;
/** Hook that will apply the style reserved for active state if needed */
const useFocus = (onFocus, onBlur, needsFocus) => {
    const [focused, setFocused] = react_1.default.useState(false);
    const focusStart = react_1.default.useMemo(() => needsFocus ? (event) => {
        if (onFocus)
            onFocus(event);
        setFocused(true);
    } : undefined, [needsFocus, onFocus]);
    const focusStop = react_1.default.useMemo(() => needsFocus ? (event) => {
        if (onBlur)
            onBlur(event);
        setFocused(false);
    } : undefined, [needsFocus, onBlur]);
    return { focused, onFocus: focusStart || onFocus, onBlur: focusStop || onBlur };
};
exports.useFocus = useFocus;
/** Hook that will apply the style provided in the media queries */
const useMediaQuery = (media, units) => {
    const mediaStyle = react_1.default.useMemo(() => {
        if (media) {
            const context = (0, mediaQueries_1.createContext)(units);
            const mediaStyles = media.map(m => m(context)).filter(m => !!m);
            if (!mediaStyles.length)
                return;
            const mq = {};
            Object.assign(mq, ...mediaStyles);
            return mq;
        }
    }, [media, units]);
    return mediaStyle;
};
exports.useMediaQuery = useMediaQuery;
/** Hook that will measure the layout to handle styles that use % units */
const useLayout = (onLayout, needsLayout) => {
    const [layout, setLayout] = react_1.default.useState({ width: 0, height: 0 });
    // Prevent calling setState if the component is unmounted
    const unmounted = react_1.default.useRef(false);
    react_1.default.useEffect(() => () => { unmounted.current = true; }, []);
    const updateLayout = react_1.default.useMemo(() => needsLayout ? (event) => {
        if (unmounted.current)
            return;
        if (onLayout)
            onLayout(event);
        const { width, height } = event.nativeEvent.layout;
        setLayout(layout => layout.width === width && layout.height === height ? layout : { width, height });
    } : undefined, [needsLayout, onLayout]);
    return { onLayout: updateLayout || onLayout, ...layout };
};
exports.useLayout = useLayout;
/** Apply the new fontSize to the component before we can calculate em units */
const useFontSize = (fontSizeTarget, rem, em) => {
    const [fontSize, fontUnit] = react_1.default.useMemo(() => fontSizeTarget === undefined ? [] : (0, convertUnits_1.parseValue)(fontSizeTarget), [fontSizeTarget]);
    const isRelative = fontUnit && ['rem', 'em', '%'].includes(fontUnit);
    const newSize = react_1.default.useMemo(() => {
        if (fontSize && isRelative) {
            const newSize = fontUnit === 'em' ? em * fontSize
                : fontUnit === 'rem' ? fontSize * rem
                    : fontUnit === '%' ? em * (1 + fontSize / 100)
                        : fontSize;
            return newSize;
        }
        else
            return fontSize || em;
    }, [em, fontSize, fontUnit, isRelative, rem]);
    return { em: newSize };
};
exports.useFontSize = useFontSize;
